# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tofh',
 'tofh.api',
 'tofh.app',
 'tofh.demo',
 'tofh.plugins',
 'tofh.plugins.mq',
 'tofh.plugins.scheduler',
 'tofh.tasks']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=2.2.2,<3.0.0',
 'PyYAML>=6.0,<7.0',
 'attrs>=22.1.0,<23.0.0',
 'blinker>=1.5,<2.0',
 'celery>=4.4.7,<5.0.0',
 'pika>=1.0.0,<2.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'python-json-logger>=2.0.4,<3.0.0',
 'pytimeparse>=1.1.8,<2.0.0',
 'redis>=4.3.4,<5.0.0',
 'sentry-sdk>=1.10.1,<2.0.0']

setup_kwargs = {
    'name': 'tofh',
    'version': '2.7.2',
    'description': '',
    'long_description': "# tofh\n\n*tofh* is a task processing and management engine, built on top of `celery`.\nIt loads third party task implementations from a configuration file, and queues\nthese tasks using extenal triggers.\n\nSee [Design](#design) for a more thorough explenation of how `tofh` works.\n\n## Install\n\nBefore installing, you should upgrade setuptools.\n\n```bash\npip install -U setuptools\n```\n\n\nTo install the module, run\n\n```bash\npip install .\n# OR\npython setup.py install\n```\n\nNote that *tofh* is designed to run on `python>=3.6`.\n\n## Configuration\n\nSee the file `example.config.yml`.\n\n## Run\n\n`TOFH_CONFIG=/path/to/tofh_config.yml python -m tofh.plugins.mq`\n\n## Run tests\n\n```bash\npython setup.py test\n```\n\n\n## Build documentation\n\n```\npython setup.py build_sphinx\n```\n\nThis will install *tofh* and all its dependencies in a temporary environment,\nand build the documentation in the `build/sphinx/html` folder.\n\nIf you already have `sphinx` and `sphinxcontrib-httpdomain`, you can also do:\n\n```bash\ncd docs\nmake html\n```\n\n\n## Try it out with Docker\n\nTODO: Make some docker entry points, and document them here.\n\n```bash\ndocker build -f Dockerfile -t tofh .\ndocker run --rm -it tofh\n```\n\n# Design\n\n`tofh` is a task processing framework of modular design, that in its most basic\nform, consists of:\n\n* a plugin that schedules tasks for processing.\n* an engine that executes scheduled tasks.\n\n## The task execution engine\n\nThe task execution engine uses [Celery](https://docs.celeryproject.org) to\npersist and execute tasks at appropriate points in time.\n\nThe following command starts the task execution engine with 16 workers:\n\n    TOFH_CONFIG=/path/to/tofh_config.yml celery worker --app=tofh.app.create --concurrency=16\n\n\nPlease see the `celery` section in [example.config.yaml](example.config.yaml)\nfor configuration of the backing store.\n\n## The MQ plugin\n\nThe MQ plugin connects to a message broker by AMQP 0.9.1, and:\n\n* Declares the appropriate exchanges\n* Declares the appropriate queues\n* Binds queues to exchanges\n* Consumes queues\n* Schedules tasks for execution\n\nThe following command starts the MQ plugin:\n\n    TOFH_CONFIG=/path/to/tofh_config.yml python -m tofh.plugins.mq\n\nSee the `tofh.plugins.mq` and `tasks` sections in\n[example.config.yaml](example.config.yaml) for a configuration example.\n\n## The Scheduler plugin\n\nThe scheduler plugin is used for handlers that does not receive notifications\n\nSchedule formats that can be used are:\n* crontab\n* timedelta\n\nThe following command starts the Scheduler plugin:\n\n    TOFH_CONFIG=/path/to/tofh_config.yml celery beat --app=tofh.plugins.scheduler.__main__\n\nSee the `tasks` sections in\n[example.config.yaml](example.config.yaml) for a configuration example.\n\n## Types of tasks\n\n`tofh` supports both stateless and stateful tasks. The latter allows for a\nstate to be preserved between invocations of a handler (e.g. keep a database or\nHTTP connection open), and can have a great effect on runtime.\n\n### Stateful tasks\n\nTo declare a task as stateful, the `task_type` attribute can be set to `stateful` for\na specific task in the `tasks` configuration.\n\nIn this mode, a context object will be passed to the handler as the first\npositional argument. The context object is mutable and persisted between\ninvocations on a per-worker basis.\n\nSee the `stateful_task` function in\n[tofh/demo/__init__.py](tofh/demo/__init__.py) for an example of how a stateful\ntask can be defined.\n\n### Stateless tasks\n\nTasks can be declared stateless by setting the `task_type` attribute to\n`stateless`, or by omitting the `task_type` attribute in the handler\nconfiguration.\n\nA context object is not passed to stateless task as the first positional argument.\n\nSee the `echo` function in [tofh/demo/__init__.py](tofh/demo/__init__.py) for\nan example.\n\n### Deciding on a task type\n\nWhen deciding if a task should be stateful, the following points might be worth considering:\n\n* 3 packets are required to set up a TCP connection\n* At least 4 transmissions are required to set up TLS\n* 1 transmission is required to tear down TLS\n* 3 packets are required to tear down a TCP connection\n* How many connections does the handler need to fetch and store information\n\nSo if the round trip time between two hosts is 10 milliseconds, the total time\nused for setup and teardown of one connection is:\n\n    (3 * 4 * 1 * 3) * (10/2) = 180 ms\n\nIf the handler executes 10 000 times, and one connection is established for\neach invocation:\n\n    (10 000 * 180 / 1 000 ) / 60 = 30 minutes\n\nThe 30 minutes spent setting up and tearing down one connection, should rather be\nused to transfer data over the connection.\n\n## Prioritizing tasks\n\nTo prioritize tasks, Celery must be configured with `task_queue_max_priority`.\nYou may also set a `task_default_priority`.\n\nFor each task that should have a different priority than the default, set its\n`priority` to a number >=0 and <=`task_queue_max_priority`. A higher number\nmeans higher priority.\n\nNote that RabbitMQ doesn't support adding arguments (in this case, `x-max-priority`)\nto a queue after it has been declared. This also applies to the task queue created\nby Celery (`task_default_queue`). You must either delete and recreate it or use\na new queue name. Use shovels to move messages if needed.\n\nFor further reading, see\n[Celery: RabbitMQ Message Priorities](https://docs.celeryproject.org/en/stable/userguide/routing.html#rabbitmq-message-priorities)\nand\n[RabbitMQ: Priority Queue Support](https://www.rabbitmq.com/priority.html).\n\n## Configuring handlers\n\nFor an example of how a handler can be configured, please see the function\n`use_a_config_variable` in [tofh/demo/__init__.py](tofh/demo/__init__.py), and\nthe corresponding `demo` attribute in\n[example.config.yaml](example.config.yaml).\n\nFor the function to find the configuration, the task execution engine should be\nstarted like:\n\n    TOFH_CONFIG=/path/to/tofh_config.yml DEMO_CONFIG=/path/to/tofh_config.yml celery worker --app=tofh.app.create --concurrency=16\n\n# Definitions\n\n* A *transform* is a function that is applied to the payload received from\n  the scheduling plugin. The application of the transform results in a set of\n  arguments.\n* A *handler* is a function that performs a specific operation (e.g. the provisioning of a user).\n* A *task* is created by the application of a set of arguments to a *handler*.\n",
    'author': 'USIT UiO',
    'author_email': 'bnt-int@usit.uio.no',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
